/*
 * SharedObject.java
 *
 * Created on 11 de Novembro de 2001, 23:56
 */

package br.shmem;

import br.shmem.util.*;
import java.io.Serializable;

/**
 *
 * @author  crolmos & tminc
 */
public class SharedObject implements Serializable {

    /** Creates new SharedObject */
    public static long defaultLeaseTime = 20000;
    
    private String globalName;
    private TimeStamp stamp = null;
    private transient boolean canWrite = true;
    private transient long lease = defaultLeaseTime;
    private transient long lastUpdated = System.currentTimeMillis();
    private String referenceClassName;
    private Serializable data;
    
    private SharedObject() {
    }

    public SharedObject(String globalName, Serializable data) {
        this.globalName = globalName;
        this.data = data;
        this.referenceClassName = "br.shmem.SharedReference";
    }
    
    public SharedObject(String globalName, Serializable data, String referenceClassName) {
        this.globalName = globalName;
        this.data = data;
        this.referenceClassName = referenceClassName;
    }
    
    public void setTimeStamp(TimeStamp stamp) {
        this.stamp = stamp;
    }
    
    public String getGlobalName() {
            return globalName;
    }
    
    public TimeStamp getTimeStamp() {
        return stamp;
    }
    
    public long getLeaseTime() {
        return lease;
    }

    public void setLeaseTime(long lease) {
	this.lease = lease;
    }
    
    protected Serializable getData() {
        return data;
    }
    
    public String getReferenceClassName() {
        return referenceClassName;
    }
    
    public boolean canWrite() {
        return canWrite;
    }
    
    public void setCanWrite(boolean flag) {
        canWrite = flag;
    }
    
    public void setLastUpdated(long time) {
        lastUpdated = time;
    }
    
    public boolean isUpdated() {
        return (System.currentTimeMillis() < lastUpdated + lease);
    }

    public SharedObjectDescriptor getDescriptor() {
	return new SharedObjectDescriptor(this);
    }
}

