/*
 * TimeStamp.java
 *
 * Created on 11 de Novembro de 2001, 19:35
 */

package br.shmem;
import java.util.Date;
import java.security.*;
/** TimeStamp is a wraper for a java.util.Date object.<br>
 * It is responsible for signatura validation, ensuring that this date was generated by the serever.<br>
 * Only the server should create <CODE>TimeStamps</CODE>.
 *
 * @author crolmos & tminc
 * @version 1.0
 */
public class TimeStamp implements java.io.Serializable {
    
    private SignedObject stamp;
    private transient Date cachedDate = null;
    
    /** Creates new TimeStamp */
    private TimeStamp() {
    }
    
    /** Main contructor for the class.
     * @param d the date object to be signed.
     * @param key the private key used for signing.
     * @throws InsecureObjectException if the signing process fails.
     */    
    public TimeStamp(Date d, PrivateKey key) 
        throws InsecureObjectException
    {
        if (d == null) throw new NullPointerException("Date cannot be null");
        try {
            stamp = new SignedObject(d, key, Signature.getInstance(AbstractCommLayer.signatureAlgorithm));
        }
        catch (Exception e) {
            throw new InsecureObjectException(e);
        }
    }
    
    /** accessor for the date object.
     * @throws InsecureObjectException if the signing process fails.
     * @return the <code>Date</code> of this time stamp.
     */    
    public Date getDate()
        throws InsecureObjectException
    {
        if (cachedDate == null)
            try {
                cachedDate = (Date) stamp.getObject();
            }
            catch (Exception e) {
                throw new InsecureObjectException(e);
            }
        return cachedDate;
    }
    
    /** verifies the autenticity of the TimeStamp.
     * @param key the public key used in the verification process.
     * @throws InsecureObjectException if the signing process fails.
     * @return <code>true</code> if the digital signature is valid, <code>false</code> otherwise.
     */    
    public boolean verify(PublicKey key) 
        throws InsecureObjectException
    {
        try {
            return stamp.verify(key, Signature.getInstance(AbstractCommLayer.signatureAlgorithm));
        }
        catch (Exception e) {
            throw new InsecureObjectException(e);
        }
    }
    
    public int compareTo(TimeStamp other) 
        throws InsecureObjectException
    {
        return getDate().compareTo(other.getDate());
    }

    public boolean before(TimeStamp other)
    throws InsecureObjectException
    {
	return getDate().before(other.getDate());
    }
    
    public String toString() {
        try {
            return getDate().toString();
        }
        catch(InsecureObjectException e) {
            return "Invalid Time Stamp";
        }
    }
}
