package br.shmem.client;

import java.net.*;
import br.shmem.*;
import java.io.*;

/**
 *
 * @author  crolmos & tminc
 */
public class SignedConnection implements Runnable {
    
    Socket socket;
    ClientCommLayer commLayer;
    HostId remoteId;
    
    public SignedConnection(ClientCommLayer commLayer, Socket socket) {
        this.socket = socket;
        this.commLayer = commLayer;
    }
    
    public void run() {
        try {
            ObjectInputStream in = new ObjectInputStream(socket.getInputStream());
            URL remoteURL = (URL)in.readObject();
            remoteId = commLayer.identify(remoteURL);
            Message m = commLayer.recvMessage(socket, remoteId);
            switch (m.getType()) {
                case Message.PING:
                    break;
                case Message.GET:
                    commLayer.sendCached(socket, m.getGlobalName());
                    socket.close();
                    break;
	    case Message.RELOAD:
		commLayer.send(socket, m.getGlobalName());
		socket.close();
		break;
	    case Message.UPDATE:
                    commLayer.update(m.getGlobalName(), m.getHostId());
                    break;
                default:
                    commLayer.sendMessage(socket, Message.createError(m.getType(), "Unrecognized message type."));
            }
        }
        catch(IOException e) {
            System.err.println(e);
            e.printStackTrace();
        }
        catch(CommLayerException e) {
            System.err.println(e);
            e.printStackTrace();
        }
        catch(ClassNotFoundException e) {
            System.err.println(e);
            e.printStackTrace();
        }
        catch(InsecureObjectException e) {
            System.err.println(e);
            e.printStackTrace();
        }
        catch(InvalidOperationException e) {
            System.err.println(e);
            e.printStackTrace();
        }
    }
}
