package br.shmem.server;

import br.shmem.*;
import java.net.URL;

/**
 *
 * @author  crolmos & tminc
 */
class SharedEntry {

    private URL owner;
    private URL writer = null;

    private TimeStamp stamp = null;
    
    private boolean lockable;

    public SharedEntry(URL owner, boolean lockable) {
	this.owner = owner;
	this.lockable = lockable;
    }
    
    public URL getOwner() {
	return owner;
    }

    public URL getWriter() {
	return writer;
    }

    public TimeStamp getTimeStamp() {
	return stamp;
    }

    public void setTimeStamp(TimeStamp stamp) {
	this.stamp = stamp;
    }

    public void lock(URL client) 
    throws AlreadyLockedException, ObjectNotLockableException {
	if (writer != null)
	    throw new AlreadyLockedException("already locked");
	if (!lockable)
	    throw new ObjectNotLockableException();
	writer = client;
    }

    public void unlock(URL client) 
	throws NotLockedException
    {
	if (writer == null || !writer.equals(client))
	    throw new NotLockedException("the client " + client + " doesn't have the lock");
	writer = null;
    }

    public boolean isLocked() {
	return (writer != null);
    }

    public void setLockable(boolean lockable) {
	this.lockable = lockable;
    }

    public boolean isLockable() {
        return lockable;
    }
}
